/*!
 * virtualized-list v1.1.0
 * MIT Licensed
 */
(function webpackUniversalModuleDefinition(root, factory) {
	if(typeof exports === 'object' && typeof module === 'object')
		module.exports = factory();
	else if(typeof define === 'function' && define.amd)
		define([], factory);
	else if(typeof exports === 'object')
		exports["VirtualizedList"] = factory();
	else
		root["VirtualizedList"] = factory();
})(this, function() {
return /******/ (function(modules) { // webpackBootstrap
/******/ 	// The module cache
/******/ 	var installedModules = {};

/******/ 	// The require function
/******/ 	function __webpack_require__(moduleId) {

/******/ 		// Check if module is in cache
/******/ 		if(installedModules[moduleId])
/******/ 			return installedModules[moduleId].exports;

/******/ 		// Create a new module (and put it into the cache)
/******/ 		var module = installedModules[moduleId] = {
/******/ 			exports: {},
/******/ 			id: moduleId,
/******/ 			loaded: false
/******/ 		};

/******/ 		// Execute the module function
/******/ 		modules[moduleId].call(module.exports, module, module.exports, __webpack_require__);

/******/ 		// Flag the module as loaded
/******/ 		module.loaded = true;

/******/ 		// Return the exports of the module
/******/ 		return module.exports;
/******/ 	}


/******/ 	// expose the modules object (__webpack_modules__)
/******/ 	__webpack_require__.m = modules;

/******/ 	// expose the module cache
/******/ 	__webpack_require__.c = installedModules;

/******/ 	// __webpack_public_path__
/******/ 	__webpack_require__.p = "";

/******/ 	// Load entry module and return exports
/******/ 	return __webpack_require__(0);
/******/ })
/************************************************************************/
/******/ ([
/* 0 */
/***/ function(module, exports, __webpack_require__) {

	module.exports = __webpack_require__(1);


/***/ },
/* 1 */
/***/ function(module, exports) {

	'use strict';

	exports.__esModule = true;

	function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

	/** Virtualized list, only renders visible items.
	 *  @param  {Array<*>} data           List of data items
	 *  @param  {Function} renderRow      Renders a single row
	 *  @param  {Number}   rowHeight      Static height of a row
	 *  @param  {Number}   overscanCount  Amount of rows to render above and below visible area of the list
	 *  @param  {Function} onMount        Callback that is invoked after the list has mounted
	 *  @method {Function} scrollToIndex  Method used to scroll to any given index
	 *
	 *  @example
	 *  new VirtualizedList(element, {
	 *    data: ['a', 'b', 'c'],
	 *    renderRow: (row) => { <div>{row}</div> },
	 *    rowHeight: 150,
	 *    overscanCount: 5
	 *  )}
	 *
	 *  TODO:
	 *  – Add support for variable heights
	 */

	var STYLE_INNER = 'position:relative; overflow:hidden; width:100%; min-height:100%;';
	var STYLE_CONTENT = 'position:absolute; top:0; left:0; height:100%; width:100%; overflow:visible;';

	var VirtualizedList = function () {
	  function VirtualizedList(container, options) {
	    _classCallCheck(this, VirtualizedList);

	    this.container = container;
	    this.options = options;

	    // Initialization
	    this.state = {};

	    // Binding
	    this.render = this.render.bind(this);
	    this.handleScroll = this.handleScroll.bind(this);
	    this.resize = this.resize.bind(this);

	    // Lifecycle Methods
	    this.componentWillMount();
	    this.componentDidMount();
	  }

	  VirtualizedList.prototype.componentWillMount = function componentWillMount() {
	    var inner = this.inner = document.createElement('div');
	    var content = this.content = document.createElement('div');

	    inner.setAttribute('style', STYLE_INNER);
	    content.setAttribute('style', STYLE_CONTENT);
	    inner.appendChild(content);
	    this.container.appendChild(inner);
	  };

	  VirtualizedList.prototype.componentDidMount = function componentDidMount() {
	    var onMount = this.options.onMount;

	    // Resize

	    this.resize(onMount);

	    // Add event listeners
	    this.container.addEventListener('scroll', this.handleScroll);
	    window.addEventListener('resize', this.resize);
	  };

	  VirtualizedList.prototype.setState = function setState() {
	    var _this = this;

	    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
	    var callback = arguments[1];

	    this.state = Object.assign(this.state, state);

	    window.requestAnimationFrame(function () {
	      _this.render();

	      if (typeof callback === 'function') {
	        callback();
	      }
	    });
	  };

	  VirtualizedList.prototype.resize = function resize(callback) {
	    if (this.state.height !== this.container.offsetHeight) {
	      this.setState({
	        height: this.container.offsetHeight
	      }, callback);
	    }
	  };

	  VirtualizedList.prototype.handleScroll = function handleScroll() {
	    this.setState({
	      offset: this.container.scrollTop
	    });
	  };

	  VirtualizedList.prototype.scrollToIndex = function scrollToIndex(index) {
	    var rowHeight = this.options.rowHeight;

	    var offset = rowHeight * index;

	    this.container.scrollTop = offset;
	  };

	  VirtualizedList.prototype.destroy = function destroy() {
	    window.removeEventListener('resize', this.resize);

	    // TODO: Destroy everything properly, silly
	  };

	  VirtualizedList.prototype.getRowsForOffset = function getRowsForOffset(offset) {
	    var _options = this.options,
	        data = _options.data,
	        _options$overscanCoun = _options.overscanCount,
	        overscanCount = _options$overscanCoun === undefined ? 3 : _options$overscanCoun,
	        rowHeight = _options.rowHeight;
	    var height = this.state.height;

	    // first visible row index

	    var start = offset / rowHeight | 0;

	    // actual number of visible rows (without overscan)
	    var visibleRowCount = height / rowHeight | 0;

	    // Overscan: render blocks of rows modulo an overscan row count
	    // This dramatically reduces DOM writes during scrolling
	    if (overscanCount) {
	      start = Math.max(0, start - start % overscanCount);
	      visibleRowCount += overscanCount;
	    }

	    // last visible + overscan row index
	    var end = start + 1 + visibleRowCount;

	    // data slice currently in viewport plus overscan items
	    return {
	      rows: data.slice(start, end),
	      start: start,
	      end: end
	    };
	  };

	  VirtualizedList.prototype.render = function render() {
	    var _options2 = this.options,
	        data = _options2.data,
	        rowHeight = _options2.rowHeight,
	        renderRow = _options2.renderRow;
	    var _state$offset = this.state.offset,
	        offset = _state$offset === undefined ? 0 : _state$offset;

	    var _getRowsForOffset = this.getRowsForOffset(offset),
	        rows = _getRowsForOffset.rows,
	        start = _getRowsForOffset.start;

	    var fragment = document.createDocumentFragment();

	    rows.forEach(function (row) {
	      return fragment.append(renderRow(row));
	    });

	    this.inner.style.height = data.length * rowHeight + 'px';
	    this.content.style.top = start * rowHeight + 'px';

	    this.content.innerHTML = '';
	    this.content.appendChild(fragment);
	  };

	  return VirtualizedList;
	}();

	exports.default = VirtualizedList;

/***/ }
/******/ ])
});
;